"""WordPress sidebar management commands"""

import click
from rich.console import Console
from rich.table import Table

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.ai_formatter import AIFormatter
from praisonaiwp.utils.logger import get_logger

console = Console()
logger = get_logger(__name__)


@click.group()
def sidebar_command():
    """Manage WordPress sidebars and widgets"""
    pass


@sidebar_command.command("list")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def list_sidebars(ctx, server, json_output):
    """
    List WordPress sidebars

    Examples:
    # List all sidebars
    praisonaiwp sidebar list

    # JSON output for scripting
    praisonaiwp --json sidebar list
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar list", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # List sidebars
        result = client.sidebar_list()

        if "error" in result:
            error_msg = result["error"]
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar list", "SIDEBAR_LIST_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(result, "sidebar list", "sidebar list")
            click.echo(AIFormatter.format_output(response))
        else:
            if not result:
                console.print("[yellow]No sidebars found[/yellow]")
                return

            table = Table(title="Sidebars")
            table.add_column("ID", style="cyan")
            table.add_column("Name", style="green")
            table.add_column("Description", style="yellow")

            for sidebar_id, sidebar_info in result.items():
                if isinstance(sidebar_info, dict):
                    name = sidebar_info.get('name', sidebar_id)
                    description = sidebar_info.get('description', '')
                else:
                    name = sidebar_info
                    description = ''

                table.add_row(sidebar_id, name, description)

            console.print(table)

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "sidebar list", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Sidebar list failed: {e}")
        raise click.Abort() from None


@sidebar_command.command("get")
@click.argument("sidebar_id")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def get_sidebar(ctx, sidebar_id, server, json_output):
    """
    Get sidebar information by ID

    Examples:
    # Get sidebar information
    praisonaiwp sidebar get "sidebar-1"

    # Get specific sidebar details
    praisonaiwp sidebar get "footer-widgets"
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar get", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Get sidebar information
        result = client.sidebar_get(sidebar_id)

        if result is None:
            error_msg = f"Sidebar '{sidebar_id}' not found"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar get", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[yellow]{error_msg}[/yellow]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(result, "sidebar get", "sidebar get")
            click.echo(AIFormatter.format_output(response))
        else:
            table = Table(title=f"Sidebar: {sidebar_id}")
            table.add_column("Property", style="cyan")
            table.add_column("Value", style="green")

            for key, value in result.items():
                if isinstance(value, list):
                    value = ", ".join(str(v) for v in value)
                table.add_row(key, str(value))

            console.print(table)

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "sidebar get", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Sidebar get failed: {e}")
        raise click.Abort() from None


@sidebar_command.command("update")
@click.argument("sidebar_id")
@click.argument("widgets", nargs=-1, required=False)
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def update_sidebar(ctx, sidebar_id, widgets, server, json_output):
    """
    Update sidebar with widgets

    Examples:
    # Update sidebar with widgets
    praisonaiwp sidebar update "sidebar-1" "widget-1" "widget-2"

    # Empty sidebar (no widgets)
    praisonaiwp sidebar update "sidebar-1"
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar update", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Update sidebar
        success = client.sidebar_update(sidebar_id, list(widgets))

        if not success:
            error_msg = "Failed to update sidebar"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar update", "SIDEBAR_UPDATE_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(
                {"sidebar_id": sidebar_id, "widgets": list(widgets)},
                "sidebar update",
                "sidebar update"
            )
            click.echo(AIFormatter.format_output(response))
        else:
            if widgets:
                console.print(f"[green]✓[/green] Sidebar '{sidebar_id}' updated with {len(widgets)} widgets")
                console.print(f"  Widgets: {', '.join(widgets)}")
            else:
                console.print(f"[green]✓[/green] Sidebar '{sidebar_id}' emptied")

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "sidebar update", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Sidebar update failed: {e}")
        raise click.Abort() from None


@sidebar_command.command("add-widget")
@click.argument("sidebar_id")
@click.argument("widget_id")
@click.option("--position", type=int, help="Position in sidebar")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def add_widget(ctx, sidebar_id, widget_id, position, server, json_output):
    """
    Add widget to sidebar

    Examples:
    # Add widget to sidebar
    praisonaiwp sidebar add-widget "sidebar-1" "search-2"

    # Add widget at specific position
    praisonaiwp sidebar add-widget "sidebar-1" "search-2" --position=1
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar add-widget", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Add widget to sidebar
        success = client.sidebar_add_widget(sidebar_id, widget_id, position)

        if not success:
            error_msg = "Failed to add widget to sidebar"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar add-widget", "SIDEBAR_ADD_WIDGET_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(
                {"sidebar_id": sidebar_id, "widget_id": widget_id, "position": position},
                "sidebar add-widget",
                "sidebar add-widget"
            )
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[green]✓[/green] Widget '{widget_id}' added to sidebar '{sidebar_id}' successfully")
            if position is not None:
                console.print(f"  Position: {position}")

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "sidebar add-widget", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Sidebar add-widget failed: {e}")
        raise click.Abort() from None


@sidebar_command.command("remove-widget")
@click.argument("sidebar_id")
@click.argument("widget_id")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def remove_widget(ctx, sidebar_id, widget_id, server, json_output):
    """
    Remove widget from sidebar

    Examples:
    # Remove widget from sidebar
    praisonaiwp sidebar remove-widget "sidebar-1" "search-2"
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar remove-widget", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Remove widget from sidebar
        success = client.sidebar_remove_widget(sidebar_id, widget_id)

        if not success:
            error_msg = "Failed to remove widget from sidebar"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar remove-widget", "SIDEBAR_REMOVE_WIDGET_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(
                {"sidebar_id": sidebar_id, "widget_id": widget_id, "removed": True},
                "sidebar remove-widget",
                "sidebar remove-widget"
            )
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[green]✓[/green] Widget '{widget_id}' removed from sidebar '{sidebar_id}' successfully")

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "sidebar remove-widget", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Sidebar remove-widget failed: {e}")
        raise click.Abort() from None


@sidebar_command.command("empty")
@click.argument("sidebar_id")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def empty_sidebar(ctx, sidebar_id, server, json_output):
    """
    Empty all widgets from sidebar

    Examples:
    # Empty sidebar
    praisonaiwp sidebar empty "sidebar-1"
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar empty", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Empty sidebar
        success = client.sidebar_empty(sidebar_id)

        if not success:
            error_msg = "Failed to empty sidebar"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "sidebar empty", "SIDEBAR_EMPTY_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(
                {"sidebar_id": sidebar_id, "emptied": True},
                "sidebar empty",
                "sidebar empty"
            )
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[green]✓[/green] Sidebar '{sidebar_id}' emptied successfully")

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "sidebar empty", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Sidebar empty failed: {e}")
        raise click.Abort() from None



"""WordPress shell commands"""
import click

from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.config import Config
from praisonaiwp.core.wp_client import WPClient


@click.command()
@click.option('--server', help='Server name from config')
def shell(server):
    """Interactive WordPress shell."""
    config = Config()
    transport = get_transport(config, server)
    transport.connect()
    client = WPClient(ssh, config.get_server(server)['wp_path'] if server else None)
    
    click.echo("Starting WordPress shell. Type 'exit' to quit.")
    
    while True:
        try:
            command = input("wp> ")
            if command.lower() in ['exit', 'quit']:
                break
            
            if command.strip():
                result = client.cli(command)
                click.echo(result)
        except KeyboardInterrupt:
            break
    
    click.echo("WordPress shell closed.")



"""WordPress taxonomy management commands"""

import click
from rich.console import Console
from rich.table import Table

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.logger import get_logger

console = Console()
logger = get_logger(__name__)


@click.group()
def taxonomy_command():
    """Manage WordPress taxonomies"""
    pass


@taxonomy_command.command("list")
@click.option("--server", default=None, help="Server name from config")
def list_taxonomies(server):
    """
    List WordPress taxonomies

    Examples:
    # List all taxonomies
    praisonaiwp taxonomy list

    # List from specific server
    praisonaiwp taxonomy list --server staging
    """
    try:
        config = Config()
        server_config = config.get_server(server) if server else config.get_default_server()

        if not server_config:
            console.print(f"[red]Server '{server}' not found in configuration[/red]")
            return

        transport = get_transport(config, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        taxonomies = client.list_taxonomies()

        if not taxonomies:
            console.print("[yellow]No taxonomies found[/yellow]")
            return

        # Create table
        table = Table(title="WordPress Taxonomies")
        table.add_column("Name", style="cyan", no_wrap=True)
        table.add_column("Label", style="green")
        table.add_column("Hierarchical", style="yellow")
        table.add_column("Public", style="blue")

        for taxonomy in taxonomies:
            table.add_row(
                taxonomy.get('name', 'N/A'),
                taxonomy.get('label', 'N/A'),
                str(taxonomy.get('hierarchical', False)),
                str(taxonomy.get('public', True))
            )

        console.print(table)

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"List taxonomies failed: {e}")
        raise click.Abort() from None


@taxonomy_command.command("get")
@click.argument("taxonomy")
@click.option("--server", default=None, help="Server name from config")
def get_taxonomy(taxonomy, server):
    """
    Get WordPress taxonomy information

    Examples:
    # Get taxonomy info
    praisonaiwp taxonomy get category

    # Get from specific server
    praisonaiwp taxonomy get post_tag --server staging
    """
    try:
        config = Config()
        server_config = config.get_server(server) if server else config.get_default_server()

        if not server_config:
            console.print(f"[red]Server '{server}' not found in configuration[/red]")
            return

        transport = get_transport(config, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        taxonomy_info = client.get_taxonomy(taxonomy)

        if taxonomy_info:
            console.print("[green]Taxonomy Information:[/green]")
            console.print(f"[cyan]Name:[/cyan] {taxonomy_info.get('name', 'N/A')}")
            console.print(f"[cyan]Label:[/cyan] {taxonomy_info.get('label', 'N/A')}")
            console.print(f"[cyan]Hierarchical:[/cyan] {taxonomy_info.get('hierarchical', False)}")
            console.print(f"[cyan]Public:[/cyan] {taxonomy_info.get('public', True)}")
            console.print(f"[cyan]Show UI:[/cyan] {taxonomy_info.get('show_ui', True)}")
        else:
            console.print(f"[yellow]Taxonomy '{taxonomy}' not found[/yellow]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Get taxonomy failed: {e}")
        raise click.Abort() from None



"""WordPress post-type management commands"""

import click
from rich.console import Console
from rich.table import Table

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.ai_formatter import AIFormatter
from praisonaiwp.utils.logger import get_logger

console = Console()
logger = get_logger(__name__)


@click.group()
def post_type_command():
    """Manage WordPress post types"""
    pass


@post_type_command.command("list")
@click.option("--format", "format_type", default="table", type=click.Choice(['table', 'json']), help="Output format")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def list_post_types(ctx, format_type, server, json_output):
    """
    List WordPress post types

    Examples:
    # List post types in table format
    praisonaiwp post-type list

    # List in JSON format
    praisonaiwp post-type list --format=json

    # JSON output for scripting
    praisonaiwp --json post-type list
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type list", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # List post types
        result = client.post_type_list(format_type)

        if "error" in result:
            error_msg = result["error"]
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type list", "POST_TYPE_LIST_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(result, "post-type list", "post-type list")
            click.echo(AIFormatter.format_output(response))
        else:
            if format_type == "json":
                console.print(result)
            else:
                if not result.get("post_types"):
                    console.print("[yellow]No post types found[/yellow]")
                    return

                table = Table(title="Post Types")
                table.add_column("Name", style="cyan")
                table.add_column("Description", style="green")

                for post_type in result["post_types"]:
                    table.add_row(post_type["name"], post_type["description"])

                console.print(table)

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "post-type list", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Post-type list failed: {e}")
        raise click.Abort() from None


@post_type_command.command("get")
@click.argument("post_type")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def get_post_type(ctx, post_type, server, json_output):
    """
    Get post type information by name

    Examples:
    # Get post type information
    praisonaiwp post-type get "post"

    # Get custom post type details
    praisonaiwp post-type get "book"
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type get", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Get post type information
        result = client.post_type_get(post_type)

        if result is None:
            error_msg = f"Post type '{post_type}' not found"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type get", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[yellow]{error_msg}[/yellow]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(result, "post-type get", "post-type get")
            click.echo(AIFormatter.format_output(response))
        else:
            table = Table(title=f"Post Type: {post_type}")
            table.add_column("Property", style="cyan")
            table.add_column("Value", style="green")

            for key, value in result.items():
                if isinstance(value, list):
                    value = ", ".join(str(v) for v in value)
                table.add_row(key, str(value))

            console.print(table)

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "post-type get", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Post-type get failed: {e}")
        raise click.Abort() from None


@post_type_command.command("create")
@click.argument("post_type")
@click.argument("label")
@click.option("--slug", help="Post type slug")
@click.option("--public", help="Whether public (true/false)")
@click.option("--has-archive", help="Whether has archive (true/false)")
@click.option("--supports", help="Supported features (comma-separated)")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def create_post_type(ctx, post_type, label, slug, public, has_archive, supports, server, json_output):
    """
    Create a new post type

    Examples:
    # Create basic post type
    praisonaiwp post-type create "book" "Books"

    # Create with options
    praisonaiwp post-type create "book" "Books" --public=true --has-archive=true

    # Create with supports
    praisonaiwp post-type create "book" "Books" --supports="title,editor,thumbnail"
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type create", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Create post type
        success = client.post_type_create(post_type, label, slug, public, has_archive, supports)

        if not success:
            error_msg = "Failed to create post type"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type create", "POST_TYPE_CREATE_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(
                {"post_type": post_type, "label": label, "slug": slug, "public": public, "has_archive": has_archive, "supports": supports},
                "post-type create",
                "post-type create"
            )
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[green]✓[/green] Post type '{post_type}' created successfully")
            console.print(f"  Label: {label}")
            if slug:
                console.print(f"  Slug: {slug}")
            if public:
                console.print(f"  Public: {public}")
            if has_archive:
                console.print(f"  Has Archive: {has_archive}")
            if supports:
                console.print(f"  Supports: {supports}")

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "post-type create", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Post-type create failed: {e}")
        raise click.Abort() from None


@post_type_command.command("delete")
@click.argument("post_type")
@click.option("--force", is_flag=True, help="Force deletion")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def delete_post_type(ctx, post_type, force, server, json_output):
    """
    Delete a post type

    Examples:
    # Delete post type
    praisonaiwp post-type delete "book"

    # Force delete
    praisonaiwp post-type delete "book" --force
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type delete", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Delete post type
        success = client.post_type_delete(post_type, force)

        if not success:
            error_msg = "Failed to delete post type"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type delete", "POST_TYPE_DELETE_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(
                {"post_type": post_type, "deleted": True, "force": force},
                "post-type delete",
                "post-type delete"
            )
            click.echo(AIFormatter.format_output(response))
        else:
            force_text = " (forced)" if force else ""
            console.print(f"[green]✓[/green] Post type '{post_type}' deleted successfully{force_text}")

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "post-type delete", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Post-type delete failed: {e}")
        raise click.Abort() from None


@post_type_command.command("update")
@click.argument("post_type")
@click.option("--label", help="New label")
@click.option("--slug", help="New slug")
@click.option("--public", help="Whether public (true/false)")
@click.option("--has-archive", help="Whether has archive (true/false)")
@click.option("--supports", help="Supported features (comma-separated)")
@click.option("--server", default=None, help="Server name from config")
@click.option("--json", "json_output", is_flag=True, help="Output in JSON format for scripting")
@click.pass_context
def update_post_type(ctx, post_type, label, slug, public, has_archive, supports, server, json_output):
    """
    Update a post type

    Examples:
    # Update label
    praisonaiwp post-type update "book" --label="New Books"

    # Update multiple properties
    praisonaiwp post-type update "book" --public=false --has-archive=true
    """
    try:
        config_manager = Config()
        server_config = config_manager.get_server(server) if server else config_manager.get_default_server()

        if not server_config:
            error_msg = f"Server '{server}' not found in configuration"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type update", "NOT_FOUND")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        transport = get_transport(config_manager, server)
        transport.connect()
        client = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )

        # Build update parameters
        update_params = {}
        if label is not None:
            update_params['label'] = label
        if slug is not None:
            update_params['slug'] = slug
        if public is not None:
            update_params['public'] = public
        if has_archive is not None:
            update_params['has_archive'] = has_archive
        if supports is not None:
            update_params['supports'] = supports

        if not update_params:
            error_msg = "No update parameters provided"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type update", "NO_PARAMS")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[yellow]{error_msg}[/yellow]")
            return

        # Update post type
        success = client.post_type_update(post_type, **update_params)

        if not success:
            error_msg = "Failed to update post type"
            if json_output or (ctx.obj and ctx.obj.get('json_output')):
                response = AIFormatter.error_response(error_msg, "post-type update", "POST_TYPE_UPDATE_ERROR")
                click.echo(AIFormatter.format_output(response))
            else:
                console.print(f"[red]{error_msg}[/red]")
            return

        # Output results
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.create_response(
                {"post_type": post_type, "updated": True, "changes": update_params},
                "post-type update",
                "post-type update"
            )
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[green]✓[/green] Post type '{post_type}' updated successfully")
            for key, value in update_params.items():
                console.print(f"  {key.title()}: {value}")

    except Exception as e:
        error_msg = str(e)
        if json_output or (ctx.obj and ctx.obj.get('json_output')):
            response = AIFormatter.error_response(error_msg, "post-type update", "CONNECTION_ERROR")
            click.echo(AIFormatter.format_output(response))
        else:
            console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Post-type update failed: {e}")
        raise click.Abort() from None



"""Tests for role CLI commands"""

from unittest.mock import Mock, patch

import pytest
from click.testing import CliRunner

from praisonaiwp.cli.commands.role import role_command


@pytest.fixture
def mock_role_wp_client():
    """Mock WPClient for role tests"""
    with patch('praisonaiwp.cli.commands.role.WPClient') as mock_client:
        client_instance = Mock()
        mock_client.return_value = client_instance
        yield client_instance


@pytest.fixture
def mock_role_ssh():
    """Mock SSHManager for role tests"""
    with patch('praisonaiwp.cli.commands.role.SSHManager') as mock_ssh:
        ssh_instance = Mock()
        mock_ssh.from_config.return_value = ssh_instance
        yield ssh_instance


@pytest.fixture
def mock_role_config():
    """Mock Config for role tests"""
    with patch('praisonaiwp.cli.commands.role.Config') as mock_config:
        config_instance = Mock()
        config_instance.get_server.return_value = {
            'hostname': 'test.com',
            'username': 'testuser',
            'key_filename': '/path/to/key',
            'wp_path': '/var/www/html'
        }
        config_instance.get_default_server.return_value = {
            'hostname': 'test.com',
            'username': 'testuser',
            'key_filename': '/path/to/key',
            'wp_path': '/var/www/html'
        }
        mock_config.return_value = config_instance
        yield config_instance


class TestRoleList:
    """Test role list command"""

    def test_role_list_success(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test successful role list"""
        runner = CliRunner()

        mock_role_wp_client.list_roles.return_value = [
            {'name': 'administrator', 'display_name': 'Administrator'},
            {'name': 'editor', 'display_name': 'Editor'},
            {'name': 'author', 'display_name': 'Author'}
        ]

        result = runner.invoke(role_command, ['list'])

        assert result.exit_code == 0
        assert 'administrator' in result.output
        assert 'editor' in result.output
        assert 'author' in result.output
        mock_role_wp_client.list_roles.assert_called_once()

    def test_role_list_empty(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role list when no roles found"""
        runner = CliRunner()

        mock_role_wp_client.list_roles.return_value = []

        result = runner.invoke(role_command, ['list'])

        assert result.exit_code == 0
        assert 'no roles' in result.output.lower()

    def test_role_list_with_server(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role list with specific server"""
        runner = CliRunner()

        mock_role_wp_client.list_roles.return_value = [{'name': 'administrator', 'display_name': 'Administrator'}]

        result = runner.invoke(role_command, ['list', '--server', 'staging'])

        assert result.exit_code == 0
        mock_role_config.get_server.assert_called_once_with('staging')

    def test_role_list_error(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role list with error"""
        runner = CliRunner()

        mock_role_wp_client.list_roles.side_effect = Exception('Connection error')

        result = runner.invoke(role_command, ['list'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()


class TestRoleGet:
    """Test role get command"""

    def test_role_get_success(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test successful role get"""
        runner = CliRunner()

        mock_role_wp_client.get_role.return_value = {
            'name': 'editor',
            'display_name': 'Editor',
            'capabilities': ['edit_posts', 'edit_pages']
        }

        result = runner.invoke(role_command, ['get', 'editor'])

        assert result.exit_code == 0
        assert 'Editor' in result.output
        assert 'edit_posts' in result.output
        mock_role_wp_client.get_role.assert_called_once_with('editor')

    def test_role_get_not_found(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role get when role not found"""
        runner = CliRunner()

        mock_role_wp_client.get_role.return_value = None

        result = runner.invoke(role_command, ['get', 'nonexistent'])

        assert result.exit_code == 0
        assert 'not found' in result.output.lower()

    def test_role_get_with_server(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role get with specific server"""
        runner = CliRunner()

        mock_role_wp_client.get_role.return_value = {'name': 'editor', 'display_name': 'Editor'}

        result = runner.invoke(role_command, ['get', 'editor', '--server', 'staging'])

        assert result.exit_code == 0
        mock_role_config.get_server.assert_called_once_with('staging')

    def test_role_get_error(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role get with error"""
        runner = CliRunner()

        mock_role_wp_client.get_role.side_effect = Exception('Connection error')

        result = runner.invoke(role_command, ['get', 'editor'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()


class TestRoleCreate:
    """Test role create command"""

    def test_role_create_success(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test successful role create"""
        runner = CliRunner()

        mock_role_wp_client.create_role.return_value = True

        result = runner.invoke(role_command, ['create', 'custom_role', 'Custom Role'])

        assert result.exit_code == 0
        assert 'success' in result.output.lower()
        assert 'Custom Role' in result.output
        mock_role_wp_client.create_role.assert_called_once_with('custom_role', 'Custom Role', None)

    def test_role_create_with_capabilities(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role create with capabilities"""
        runner = CliRunner()

        mock_role_wp_client.create_role.return_value = True

        result = runner.invoke(role_command, ['create', 'custom_role', 'Custom Role', '--capabilities', 'edit_posts,edit_pages'])

        assert result.exit_code == 0
        mock_role_wp_client.create_role.assert_called_once_with('custom_role', 'Custom Role', 'edit_posts,edit_pages')

    def test_role_create_with_server(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role create with specific server"""
        runner = CliRunner()

        mock_role_wp_client.create_role.return_value = True

        result = runner.invoke(role_command, ['create', 'custom_role', 'Custom Role', '--server', 'staging'])

        assert result.exit_code == 0
        mock_role_config.get_server.assert_called_once_with('staging')

    def test_role_create_failure(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role create when operation fails"""
        runner = CliRunner()

        mock_role_wp_client.create_role.return_value = False

        result = runner.invoke(role_command, ['create', 'custom_role', 'Custom Role'])

        assert result.exit_code == 1
        assert 'failed' in result.output.lower()

    def test_role_create_error(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role create with error"""
        runner = CliRunner()

        mock_role_wp_client.create_role.side_effect = Exception('Creation error')

        result = runner.invoke(role_command, ['create', 'custom_role', 'Custom Role'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()


class TestRoleDelete:
    """Test role delete command"""

    def test_role_delete_success(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test successful role delete"""
        runner = CliRunner()

        mock_role_wp_client.delete_role.return_value = True

        result = runner.invoke(role_command, ['delete', 'custom_role'])

        assert result.exit_code == 0
        assert 'success' in result.output.lower()
        mock_role_wp_client.delete_role.assert_called_once_with('custom_role')

    def test_role_delete_with_server(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role delete with specific server"""
        runner = CliRunner()

        mock_role_wp_client.delete_role.return_value = True

        result = runner.invoke(role_command, ['delete', 'custom_role', '--server', 'staging'])

        assert result.exit_code == 0
        mock_role_config.get_server.assert_called_once_with('staging')

    def test_role_delete_failure(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role delete when operation fails"""
        runner = CliRunner()

        mock_role_wp_client.delete_role.return_value = False

        result = runner.invoke(role_command, ['delete', 'custom_role'])

        assert result.exit_code == 1
        assert 'failed' in result.output.lower()

    def test_role_delete_error(self, mock_role_wp_client, mock_role_ssh, mock_role_config):
        """Test role delete with error"""
        runner = CliRunner()

        mock_role_wp_client.delete_role.side_effect = Exception('Deletion error')

        result = runner.invoke(role_command, ['delete', 'custom_role'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()



"""Plugin management commands"""

import click

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.logger import get_logger

logger = get_logger(__name__)


@click.group()
def plugin():
    """Manage WordPress plugins"""
    pass


@plugin.command('list')
@click.option('--status', type=click.Choice(['all', 'active', 'inactive']), default='all',
              help='Filter plugins by status')
@click.option('--server', help='Server name from config (default: first server)')
def list_plugins(status, server):
    """
    List installed WordPress plugins

    Examples:
        # List all plugins
        praisonaiwp plugin list

        # List only active plugins
        praisonaiwp plugin list --status active
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        kwargs = {}
        if status != 'all':
            kwargs['status'] = status

        plugins = wp.list_plugins(**kwargs)

        if not plugins:
            click.echo("No plugins found")
            return

        click.echo(f"\nFound {len(plugins)} plugin(s):\n")
        for plugin in plugins:
            name = plugin.get('name', 'Unknown')
            status_str = plugin.get('status', 'unknown')
            version = plugin.get('version', 'N/A')
            update = plugin.get('update', 'none')

            status_icon = "✓" if status_str == "active" else "○"
            update_icon = " [UPDATE AVAILABLE]" if update != "none" else ""

            click.echo(f"  {status_icon} {name} (v{version}){update_icon}")
            click.echo(f"    Status: {status_str}")
            if update != "none":
                click.echo(f"    Update: {update}")
            click.echo()

    except Exception as e:
        logger.error(f"Failed to list plugins: {e}")
        click.echo(f"Error: {e}", err=True)
        raise click.Abort() from None


@plugin.command('update')
@click.argument('plugin', default='all')
@click.option('--server', help='Server name from config (default: first server)')
def update_plugin(plugin, server):
    """
    Update WordPress plugin(s)

    PLUGIN: Plugin slug/path or "all" to update all plugins (default: all)

    Examples:
        # Update all plugins
        praisonaiwp plugin update
        praisonaiwp plugin update all

        # Update specific plugin
        praisonaiwp plugin update akismet
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        click.echo(f"Updating plugin(s): {plugin}...")
        wp.update_plugin(plugin)
        click.echo(f"✓ Successfully updated plugin(s): {plugin}")

    except Exception as e:
        logger.error(f"Failed to update plugin: {e}")
        click.echo(f"Error: {e}", err=True)
        raise click.Abort() from None


@plugin.command('activate')
@click.argument('plugin')
@click.option('--server', help='Server name from config (default: first server)')
def activate_plugin(plugin, server):
    """
    Activate a WordPress plugin

    Examples:
        praisonaiwp plugin activate akismet
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        click.echo(f"Activating plugin: {plugin}...")
        wp.activate_plugin(plugin)
        click.echo(f"✓ Successfully activated plugin: {plugin}")

    except Exception as e:
        logger.error(f"Failed to activate plugin: {e}")
        click.echo(f"Error: {e}", err=True)
        raise click.Abort() from None


@plugin.command('deactivate')
@click.argument('plugin')
@click.option('--server', help='Server name from config (default: first server)')
def deactivate_plugin(plugin, server):
    """
    Deactivate a WordPress plugin

    Examples:
        praisonaiwp plugin deactivate akismet
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        click.echo(f"Deactivating plugin: {plugin}...")
        wp.deactivate_plugin(plugin)
        click.echo(f"✓ Successfully deactivated plugin: {plugin}")

    except Exception as e:
        logger.error(f"Failed to deactivate plugin: {e}")
        click.echo(f"Error: {e}", err=True)
        raise click.Abort() from None


@plugin.command('install')
@click.argument('plugin')
@click.option('--version', help='Specific version to install')
@click.option('--force', is_flag=True, help='Force installation even if already installed')
@click.option('--server', help='Server name from config (default: first server)')
def install_plugin(plugin, version, force, server):
    """
    Install a WordPress plugin

    Examples:
        # Install latest version
        praisonaiwp plugin install akismet

        # Install specific version
        praisonaiwp plugin install akismet --version 5.0

        # Force installation
        praisonaiwp plugin install akismet --force
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        click.echo(f"Installing plugin: {plugin}...")
        success = wp.plugin_install(plugin, version, force)
        if success:
            click.echo(f"✓ Successfully installed plugin: {plugin}")
        else:
            click.echo(f"✗ Failed to install plugin: {plugin}")
            raise click.Abort()

    except Exception as e:
        logger.error(f"Failed to install plugin: {e}")
        click.echo(f"Error: {e}", err=True)
        raise click.Abort() from None


@plugin.command('delete')
@click.argument('plugin')
@click.option('--deactivate/--no-deactivate', default=True, help='Deactivate before deleting')
@click.option('--server', help='Server name from config (default: first server)')
def delete_plugin(plugin, deactivate, server):
    """
    Delete a WordPress plugin

    Examples:
        # Delete plugin (deactivates first)
        praisonaiwp plugin delete akismet

        # Delete without deactivating
        praisonaiwp plugin delete akismet --no-deactivate
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        click.echo(f"Deleting plugin: {plugin}...")
        success = wp.plugin_delete(plugin, deactivate)
        if success:
            click.echo(f"✓ Successfully deleted plugin: {plugin}")
        else:
            click.echo(f"✗ Failed to delete plugin: {plugin}")
            raise click.Abort()

    except Exception as e:
        logger.error(f"Failed to delete plugin: {e}")
        click.echo(f"Error: {e}", err=True)
        raise click.Abort() from None



"""Tests for post utility CLI commands"""

from unittest.mock import MagicMock, patch

import pytest
from click.testing import CliRunner

from praisonaiwp.cli.commands.db import db_command
from praisonaiwp.cli.commands.post import post_command


@pytest.fixture
def runner():
    return CliRunner()


@pytest.fixture
def mock_wp_client():
    with patch("praisonaiwp.cli.commands.post.WPClient") as mock:
        client = MagicMock()
        mock.return_value = client
        yield client


@pytest.fixture
def mock_db_wp_client():
    with patch("praisonaiwp.cli.commands.db.WPClient") as mock:
        client = MagicMock()
        mock.return_value = client
        yield client


@pytest.fixture
def mock_ssh():
    with patch("praisonaiwp.cli.commands.post.SSHManager") as mock:
        yield mock


@pytest.fixture
def mock_db_ssh():
    with patch("praisonaiwp.cli.commands.db.SSHManager") as mock:
        yield mock


@pytest.fixture
def mock_config():
    with patch("praisonaiwp.cli.commands.post.Config") as mock:
        config = MagicMock()
        config.get_server.return_value = {
            "hostname": "test.com",
            "username": "testuser",
            "wp_path": "/var/www/html",
            "php_bin": "php",
            "wp_cli": "/usr/local/bin/wp",
        }
        mock.return_value = config
        yield config


@pytest.fixture
def mock_db_config():
    with patch("praisonaiwp.cli.commands.db.Config") as mock:
        config = MagicMock()
        config.get_server.return_value = {
            "hostname": "test.com",
            "username": "testuser",
            "wp_path": "/var/www/html",
            "php_bin": "php",
            "wp_cli": "/usr/local/bin/wp",
        }
        mock.return_value = config
        yield config


class TestPostDelete:
    def test_post_delete_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test deleting a post"""
        mock_wp_client.delete_post.return_value = True

        result = runner.invoke(post_command, ["delete", "123"], input="y\n")

        assert result.exit_code == 0
        assert "Deleted" in result.output or "✓" in result.output
        mock_wp_client.delete_post.assert_called_once_with(123)

    def test_post_delete_cancelled(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test cancelling post deletion"""
        result = runner.invoke(post_command, ["delete", "123"], input="n\n")

        assert result.exit_code == 1
        mock_wp_client.delete_post.assert_not_called()


class TestPostExists:
    def test_post_exists_true(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test checking if post exists (true)"""
        mock_wp_client.post_exists.return_value = True

        result = runner.invoke(post_command, ["exists", "123"])

        assert result.exit_code == 0
        assert "exists" in result.output or "✓" in result.output
        mock_wp_client.post_exists.assert_called_once_with(123)

    def test_post_exists_false(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test checking if post exists (false)"""
        mock_wp_client.post_exists.return_value = False

        result = runner.invoke(post_command, ["exists", "999"])

        assert result.exit_code == 1
        assert "does not exist" in result.output or "✗" in result.output


class TestDbQuery:
    def test_db_query_success(self, runner, mock_db_config, mock_db_ssh, mock_db_wp_client):
        """Test database query"""
        mock_db_wp_client.db_query.return_value = [
            {"ID": "123", "post_title": "Test Post", "post_status": "publish"},
            {"ID": "456", "post_title": "Another Post", "post_status": "draft"},
        ]

        result = runner.invoke(
            db_command, ["query", "SELECT * FROM wp_posts WHERE post_status = 'publish'"]
        )

        assert result.exit_code == 0
        mock_db_wp_client.db_query.assert_called_once_with(
            "SELECT * FROM wp_posts WHERE post_status = 'publish'"
        )

    def test_db_query_empty(self, runner, mock_db_config, mock_db_ssh, mock_db_wp_client):
        """Test database query with no results"""
        mock_db_wp_client.db_query.return_value = []

        result = runner.invoke(
            db_command, ["query", "SELECT * FROM wp_posts WHERE post_title = 'nonexistent'"]
        )

        assert result.exit_code == 0
        assert "No results" in result.output

    def test_db_query_with_server(self, runner, mock_db_config, mock_db_ssh, mock_db_wp_client):
        """Test database query with specific server"""
        mock_db_wp_client.db_query.return_value = [{"ID": "123"}]

        result = runner.invoke(
            db_command, ["query", "SELECT COUNT(*) FROM wp_posts", "--server", "production"]
        )

        assert result.exit_code == 0
        mock_db_config.get_server.assert_called_with("production")



"""Tests for scaffold CLI commands"""

from unittest.mock import Mock, patch

import pytest
from click.testing import CliRunner

from praisonaiwp.cli.commands.scaffold import scaffold_command


@pytest.fixture
def mock_scaffold_wp_client():
    """Mock WPClient for scaffold tests"""
    with patch('praisonaiwp.cli.commands.scaffold.WPClient') as mock_client:
        client_instance = Mock()
        mock_client.return_value = client_instance
        yield client_instance


@pytest.fixture
def mock_scaffold_ssh():
    """Mock SSHManager for scaffold tests"""
    with patch('praisonaiwp.cli.commands.scaffold.SSHManager') as mock_ssh:
        ssh_instance = Mock()
        mock_ssh.from_config.return_value = ssh_instance
        yield ssh_instance


@pytest.fixture
def mock_scaffold_config():
    """Mock Config for scaffold tests"""
    with patch('praisonaiwp.cli.commands.scaffold.Config') as mock_config:
        config_instance = Mock()
        config_instance.get_server.return_value = {
            'hostname': 'test.com',
            'username': 'testuser',
            'key_filename': '/path/to/key',
            'wp_path': '/var/www/html'
        }
        config_instance.get_default_server.return_value = {
            'hostname': 'test.com',
            'username': 'testuser',
            'key_filename': '/path/to/key',
            'wp_path': '/var/www/html'
        }
        mock_config.return_value = config_instance
        yield config_instance


class TestScaffoldPostType:
    """Test scaffold post-type command"""

    def test_scaffold_post_type_success(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test successful post type scaffold"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_post_type.return_value = True

        result = runner.invoke(scaffold_command, ['post-type', 'book'])

        assert result.exit_code == 0
        assert 'success' in result.output.lower()
        assert 'book' in result.output
        mock_scaffold_wp_client.scaffold_post_type.assert_called_once_with('book', None, None, None, None)

    def test_scaffold_post_type_with_options(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test post type scaffold with options"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_post_type.return_value = True

        result = runner.invoke(scaffold_command, [
            'post-type', 'book',
            '--label', 'Books',
            '--public', 'true',
            '--has_archive', 'true',
            '--supports', 'title,editor'
        ])

        assert result.exit_code == 0
        mock_scaffold_wp_client.scaffold_post_type.assert_called_once_with(
            'book', 'Books', 'true', 'true', 'title,editor'
        )

    def test_scaffold_post_type_with_server(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test post type scaffold with specific server"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_post_type.return_value = True

        result = runner.invoke(scaffold_command, ['post-type', 'book', '--server', 'staging'])

        assert result.exit_code == 0
        mock_scaffold_config.get_server.assert_called_once_with('staging')

    def test_scaffold_post_type_failure(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test post type scaffold when operation fails"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_post_type.return_value = False

        result = runner.invoke(scaffold_command, ['post-type', 'book'])

        assert result.exit_code == 1
        assert 'failed' in result.output.lower()

    def test_scaffold_post_type_error(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test post type scaffold with error"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_post_type.side_effect = Exception('Scaffold error')

        result = runner.invoke(scaffold_command, ['post-type', 'book'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()


class TestScaffoldTaxonomy:
    """Test scaffold taxonomy command"""

    def test_scaffold_taxonomy_success(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test successful taxonomy scaffold"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_taxonomy.return_value = True

        result = runner.invoke(scaffold_command, ['taxonomy', 'genre'])

        assert result.exit_code == 0
        assert 'success' in result.output.lower()
        assert 'genre' in result.output
        mock_scaffold_wp_client.scaffold_taxonomy.assert_called_once_with('genre', None, None, None, None)

    def test_scaffold_taxonomy_with_options(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test taxonomy scaffold with options"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_taxonomy.return_value = True

        result = runner.invoke(scaffold_command, [
            'taxonomy', 'genre',
            '--label', 'Genres',
            '--public', 'true',
            '--hierarchical', 'true',
            '--post_types', 'book'
        ])

        assert result.exit_code == 0
        mock_scaffold_wp_client.scaffold_taxonomy.assert_called_once_with(
            'genre', 'Genres', 'true', 'true', 'book'
        )

    def test_scaffold_taxonomy_with_server(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test taxonomy scaffold with specific server"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_taxonomy.return_value = True

        result = runner.invoke(scaffold_command, ['taxonomy', 'genre', '--server', 'staging'])

        assert result.exit_code == 0
        mock_scaffold_config.get_server.assert_called_once_with('staging')

    def test_scaffold_taxonomy_failure(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test taxonomy scaffold when operation fails"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_taxonomy.return_value = False

        result = runner.invoke(scaffold_command, ['taxonomy', 'genre'])

        assert result.exit_code == 1
        assert 'failed' in result.output.lower()

    def test_scaffold_taxonomy_error(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test taxonomy scaffold with error"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_taxonomy.side_effect = Exception('Scaffold error')

        result = runner.invoke(scaffold_command, ['taxonomy', 'genre'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()


class TestScaffoldPlugin:
    """Test scaffold plugin command"""

    def test_scaffold_plugin_success(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test successful plugin scaffold"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_plugin.return_value = True

        result = runner.invoke(scaffold_command, ['plugin', 'my-plugin'])

        assert result.exit_code == 0
        assert 'success' in result.output.lower()
        assert 'my-plugin' in result.output
        mock_scaffold_wp_client.scaffold_plugin.assert_called_once_with('my-plugin', None, None, None)

    def test_scaffold_plugin_with_options(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test plugin scaffold with options"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_plugin.return_value = True

        result = runner.invoke(scaffold_command, [
            'plugin', 'my-plugin',
            '--plugin_name', 'My Plugin',
            '--plugin_uri', 'https://example.com',
            '--author', 'Test Author'
        ])

        assert result.exit_code == 0
        mock_scaffold_wp_client.scaffold_plugin.assert_called_once_with(
            'my-plugin', 'My Plugin', 'https://example.com', 'Test Author'
        )

    def test_scaffold_plugin_with_server(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test plugin scaffold with specific server"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_plugin.return_value = True

        result = runner.invoke(scaffold_command, ['plugin', 'my-plugin', '--server', 'staging'])

        assert result.exit_code == 0
        mock_scaffold_config.get_server.assert_called_once_with('staging')

    def test_scaffold_plugin_failure(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test plugin scaffold when operation fails"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_plugin.return_value = False

        result = runner.invoke(scaffold_command, ['plugin', 'my-plugin'])

        assert result.exit_code == 1
        assert 'failed' in result.output.lower()

    def test_scaffold_plugin_error(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test plugin scaffold with error"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_plugin.side_effect = Exception('Scaffold error')

        result = runner.invoke(scaffold_command, ['plugin', 'my-plugin'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()


class TestScaffoldTheme:
    """Test scaffold theme command"""

    def test_scaffold_theme_success(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test successful theme scaffold"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_theme.return_value = True

        result = runner.invoke(scaffold_command, ['theme', 'my-theme'])

        assert result.exit_code == 0
        assert 'success' in result.output.lower()
        assert 'my-theme' in result.output
        mock_scaffold_wp_client.scaffold_theme.assert_called_once_with('my-theme', None, None, None, None)

    def test_scaffold_theme_with_options(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test theme scaffold with options"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_theme.return_value = True

        result = runner.invoke(scaffold_command, [
            'theme', 'my-theme',
            '--theme_name', 'My Theme',
            '--theme_uri', 'https://example.com',
            '--author', 'Test Author',
            '--author_uri', 'https://author.com'
        ])

        assert result.exit_code == 0
        mock_scaffold_wp_client.scaffold_theme.assert_called_once_with(
            'my-theme', 'My Theme', 'https://example.com', 'Test Author', 'https://author.com'
        )

    def test_scaffold_theme_with_server(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test theme scaffold with specific server"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_theme.return_value = True

        result = runner.invoke(scaffold_command, ['theme', 'my-theme', '--server', 'staging'])

        assert result.exit_code == 0
        mock_scaffold_config.get_server.assert_called_once_with('staging')

    def test_scaffold_theme_failure(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test theme scaffold when operation fails"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_theme.return_value = False

        result = runner.invoke(scaffold_command, ['theme', 'my-theme'])

        assert result.exit_code == 1
        assert 'failed' in result.output.lower()

    def test_scaffold_theme_error(self, mock_scaffold_wp_client, mock_scaffold_ssh, mock_scaffold_config):
        """Test theme scaffold with error"""
        runner = CliRunner()

        mock_scaffold_wp_client.scaffold_theme.side_effect = Exception('Scaffold error')

        result = runner.invoke(scaffold_command, ['theme', 'my-theme'])

        assert result.exit_code == 1
        assert 'error' in result.output.lower()



"""User management commands"""

import json

import click
from rich.console import Console
from rich.table import Table

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.logger import get_logger

console = Console()
logger = get_logger(__name__)


@click.group()
def user_command():
    """Manage WordPress users"""
    pass


@user_command.command('list')
@click.option('--role', help='Filter by role (administrator, editor, author, etc.)')
@click.option('--server', default=None, help='Server name from config')
def list_users(role, server):
    """
    List WordPress users

    Examples:

        # List all users
        praisonaiwp user list

        # List administrators only
        praisonaiwp user list --role administrator
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        filters = {}
        if role:
            filters['role'] = role

        users = wp.list_users(**filters)

        if not users:
            console.print("[yellow]No users found[/yellow]")
            return

        table = Table(title="WordPress Users")
        table.add_column("ID", style="cyan")
        table.add_column("Username", style="green")
        table.add_column("Email", style="blue")
        table.add_column("Role", style="yellow")

        for user in users:
            table.add_row(
                str(user.get('ID', '')),
                user.get('user_login', ''),
                user.get('user_email', ''),
                user.get('roles', '')
            )

        console.print(table)
        console.print(f"\n[dim]Total: {len(users)} user(s)[/dim]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"List users failed: {e}")
        raise click.Abort() from None


@user_command.command('get')
@click.argument('user_id', type=int)
@click.option('--server', default=None, help='Server name from config')
def get_user(user_id, server):
    """
    Get user details

    Examples:

        # Get user info
        praisonaiwp user get 1
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        user = wp.get_user(user_id)
        console.print_json(json.dumps(user, indent=2))

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Get user failed: {e}")
        raise click.Abort() from None


@user_command.command('create')
@click.argument('username')
@click.argument('email')
@click.option('--role', default='subscriber', help='User role')
@click.option('--password', help='User password')
@click.option('--first-name', help='First name')
@click.option('--last-name', help='Last name')
@click.option('--server', default=None, help='Server name from config')
def create_user(username, email, role, password, first_name, last_name, server):
    """
    Create a new user

    Examples:

        # Create user
        praisonaiwp user create john john@example.com --role editor

        # Create with password
        praisonaiwp user create jane jane@example.com --password secret123
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        kwargs = {'role': role}
        if password:
            kwargs['user_pass'] = password
        if first_name:
            kwargs['first_name'] = first_name
        if last_name:
            kwargs['last_name'] = last_name

        user_id = wp.create_user(username, email, **kwargs)
        console.print(f"[green]✓ Created user ID: {user_id}[/green]")
        console.print(f"Username: {username}")
        console.print(f"Email: {email}")
        console.print(f"Role: {role}")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Create user failed: {e}")
        raise click.Abort() from None


@user_command.command('update')
@click.argument('user_id', type=int)
@click.option('--email', help='New email')
@click.option('--role', help='New role')
@click.option('--password', help='New password')
@click.option('--first-name', help='First name')
@click.option('--last-name', help='Last name')
@click.option('--server', default=None, help='Server name from config')
def update_user(user_id, email, role, password, first_name, last_name, server):
    """
    Update user

    Examples:

        # Update email
        praisonaiwp user update 5 --email newemail@example.com

        # Change role
        praisonaiwp user update 5 --role editor
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        kwargs = {}
        if email:
            kwargs['user_email'] = email
        if role:
            kwargs['role'] = role
        if password:
            kwargs['user_pass'] = password
        if first_name:
            kwargs['first_name'] = first_name
        if last_name:
            kwargs['last_name'] = last_name

        if not kwargs:
            console.print("[yellow]No fields to update[/yellow]")
            return

        wp.update_user(user_id, **kwargs)
        console.print(f"[green]✓ Updated user ID: {user_id}[/green]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Update user failed: {e}")
        raise click.Abort() from None


@user_command.command('delete')
@click.argument('user_id', type=int)
@click.option('--reassign', type=int, help='Reassign posts to this user ID')
@click.option('--server', default=None, help='Server name from config')
@click.confirmation_option(prompt='Are you sure you want to delete this user?')
def delete_user(user_id, reassign, server):
    """
    Delete user

    Examples:

        # Delete user
        praisonaiwp user delete 5

        # Delete and reassign posts
        praisonaiwp user delete 5 --reassign 1
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        wp.delete_user(user_id, reassign=reassign)
        console.print(f"[green]✓ Deleted user ID: {user_id}[/green]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Delete user failed: {e}")
        raise click.Abort() from None



"""Category command - Manage WordPress post categories"""

import click
from rich.console import Console
from rich.table import Table

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.logger import get_logger

console = Console()
logger = get_logger(__name__)


def _parse_category_input(category_str, category_id_str, wp):
    """
    Parse category input and return list of category IDs

    Args:
        category_str: Comma-separated category names/slugs
        category_id_str: Comma-separated category IDs
        wp: WPClient instance

    Returns:
        List of category IDs
    """
    category_ids = []

    if category_id_str:
        # Parse category IDs
        try:
            category_ids = [int(cid.strip()) for cid in category_id_str.split(",")]
        except ValueError as err:
            raise click.ClickException(
                "Invalid category ID format. Must be comma-separated integers."
            ) from err

    elif category_str:
        # Parse category names/slugs
        category_names = [name.strip() for name in category_str.split(",")]

        for name in category_names:
            cat = wp.get_category_by_name(name)
            if not cat:
                raise click.ClickException(f"Category '{name}' not found")
            category_ids.append(int(cat["term_id"]))

    else:
        raise click.ClickException("Either --category or --category-id must be provided")

    return category_ids


@click.group()
def category_command():
    """Manage WordPress post categories"""
    pass


@category_command.command(name="set")
@click.argument("post_id", type=int)
@click.option("--category", help="Comma-separated category names/slugs")
@click.option("--category-id", help="Comma-separated category IDs")
@click.option("--server", default=None, help="Server name from config")
def set_categories(post_id, category, category_id, server):
    """
    Set post categories (replace all existing)

    Examples:

        # Set by category name
        praisonaiwp category set 48975 --category "RAG"

        # Set multiple categories
        praisonaiwp category set 48975 --category "RAG,AI,Python"

        # Set by category ID
        praisonaiwp category set 48975 --category-id "353"
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        console.print(f"\n[yellow]Setting categories for post {post_id}...[/yellow]")

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        # Parse category input
        category_ids = _parse_category_input(category, category_id, wp)

        # Get category names for display
        category_names = []
        for cid in category_ids:
            cat = wp.get_category_by_id(cid)
            if cat:
                category_names.append(cat["name"])

        # Set categories
        wp.set_post_categories(post_id, category_ids)

        console.print(
            f"[green]✓ Set categories for post {post_id}: {', '.join(category_names)}[/green]\n"
        )

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Category set failed: {e}")
        raise click.Abort() from None


@category_command.command(name="add")
@click.argument("post_id", type=int)
@click.option("--category", help="Comma-separated category names/slugs")
@click.option("--category-id", help="Comma-separated category IDs")
@click.option("--server", default=None, help="Server name from config")
def add_categories(post_id, category, category_id, server):
    """
    Add categories to post (append)

    Examples:

        # Add by category name
        praisonaiwp category add 48975 --category "Python"

        # Add multiple categories
        praisonaiwp category add 48975 --category "Python,Tutorial"

        # Add by category ID
        praisonaiwp category add 48975 --category-id "42"
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        console.print(f"\n[yellow]Adding categories to post {post_id}...[/yellow]")

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        # Parse category input
        category_ids = _parse_category_input(category, category_id, wp)

        # Add each category
        added_names = []
        for cid in category_ids:
            wp.add_post_category(post_id, cid)
            cat = wp.get_category_by_id(cid)
            if cat:
                added_names.append(cat["name"])

        console.print(
            f"[green]✓ Added categories to post {post_id}: {', '.join(added_names)}[/green]\n"
        )

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Category add failed: {e}")
        raise click.Abort() from None


@category_command.command(name="remove")
@click.argument("post_id", type=int)
@click.option("--category", help="Comma-separated category names/slugs")
@click.option("--category-id", help="Comma-separated category IDs")
@click.option("--server", default=None, help="Server name from config")
def remove_categories(post_id, category, category_id, server):
    """
    Remove categories from post

    Examples:

        # Remove by category name
        praisonaiwp category remove 48975 --category "Uncategorized"

        # Remove multiple categories
        praisonaiwp category remove 48975 --category "Draft,Uncategorized"

        # Remove by category ID
        praisonaiwp category remove 48975 --category-id "1"
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        console.print(f"\n[yellow]Removing categories from post {post_id}...[/yellow]")

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        # Parse category input
        category_ids = _parse_category_input(category, category_id, wp)

        # Remove each category
        removed_names = []
        for cid in category_ids:
            cat = wp.get_category_by_id(cid)
            if cat:
                removed_names.append(cat["name"])
            wp.remove_post_category(post_id, cid)

        console.print(
            f"[green]✓ Removed categories from post {post_id}: {', '.join(removed_names)}[/green]\n"
        )

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Category remove failed: {e}")
        raise click.Abort() from None


@category_command.command(name="list")
@click.argument("post_id", type=int, required=False)
@click.option("--search", help="Search query")
@click.option("--server", default=None, help="Server name from config")
def list_categories(post_id, search, server):
    """
    List categories (all or for specific post)

    Examples:

        # List all categories
        praisonaiwp category list

        # List categories for specific post
        praisonaiwp category list 48975

        # Search categories
        praisonaiwp category list --search "RAG"
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        if post_id:
            # List categories for specific post
            console.print(f"\n[yellow]Fetching categories for post {post_id}...[/yellow]\n")
            categories = wp.get_post_categories(post_id)
            title = f"Categories for Post {post_id}"
        else:
            # List all categories
            if search:
                console.print(f"\n[yellow]Searching categories for '{search}'...[/yellow]\n")
            else:
                console.print("\n[yellow]Fetching all categories...[/yellow]\n")
            categories = wp.list_categories(search=search)
            title = "All Categories" if not search else f"Categories matching '{search}'"

        if not categories:
            console.print("[yellow]No categories found[/yellow]\n")
            return

        # Display results in table
        table = Table(title=f"{title} ({len(categories)})")

        table.add_column("ID", style="cyan", no_wrap=True)
        table.add_column("Name", style="white")
        table.add_column("Slug", style="dim")
        table.add_column("Parent", style="yellow")

        if not post_id:
            table.add_column("Count", style="green")

        for cat in categories:
            row = [str(cat["term_id"]), cat["name"], cat["slug"], str(cat.get("parent", "0"))]

            if not post_id:
                row.append(str(cat.get("count", "0")))

            table.add_row(*row)

        console.print(table)
        console.print()

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Category list failed: {e}")
        raise click.Abort() from None


@category_command.command(name="search")
@click.argument("query")
@click.option("--server", default=None, help="Server name from config")
def search_categories(query, server):
    """
    Search for categories by name

    Examples:

        # Search for categories
        praisonaiwp category search "RAG"

        # Search for AI categories
        praisonaiwp category search "AI"
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        console.print(f"\n[yellow]Searching for categories matching '{query}'...[/yellow]\n")

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        categories = wp.list_categories(search=query)

        if not categories:
            console.print(f"[yellow]No categories found matching '{query}'[/yellow]\n")
            return

        # Display results in table
        table = Table(title=f"Categories matching '{query}' ({len(categories)})")

        table.add_column("ID", style="cyan", no_wrap=True)
        table.add_column("Name", style="white")
        table.add_column("Slug", style="dim")
        table.add_column("Parent", style="yellow")
        table.add_column("Count", style="green")

        for cat in categories:
            table.add_row(
                str(cat["term_id"]),
                cat["name"],
                cat["slug"],
                str(cat.get("parent", "0")),
                str(cat.get("count", "0")),
            )

        console.print(table)
        console.print()

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Category search failed: {e}")
        raise click.Abort() from None


@category_command.command(name="create")
@click.argument("name")
@click.option("--slug", help="Category slug")
@click.option("--description", help="Category description")
@click.option("--parent", type=int, help="Parent category ID")
@click.option("--server", default=None, help="Server name from config")
def create_category(name, slug, description, parent, server):
    """
    Create a new category

    Examples:

        # Create basic category
        praisonaiwp category create "Technology"

        # Create category with slug and description
        praisonaiwp category create "AI" --slug "artificial-intelligence" --description "AI related posts"

        # Create child category
        praisonaiwp category create "Machine Learning" --parent 123
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        category_id = wp.create_category(
            name=name, slug=slug, description=description, parent=parent
        )

        console.print(f"[green]✓ Created category '{name}' with ID {category_id}[/green]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Create category failed: {e}")
        raise click.Abort() from None


@category_command.command(name="update")
@click.argument("category_id", type=int)
@click.option("--name", help="New category name")
@click.option("--slug", help="New category slug")
@click.option("--description", help="New category description")
@click.option("--parent", type=int, help="New parent category ID")
@click.option("--server", default=None, help="Server name from config")
def update_category(category_id, name, slug, description, parent, server):
    """
    Update an existing category

    Examples:

        # Update category name
        praisonaiwp category update 123 --name "New Name"

        # Update multiple fields
        praisonaiwp category update 123 --name "AI" --slug "artificial-intelligence" --description "Updated description"
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        success = wp.update_category(
            category_id=category_id,
            name=name,
            slug=slug,
            description=description,
            parent=parent,
        )

        if success:
            console.print(f"[green]✓ Updated category {category_id}[/green]")
        else:
            console.print(f"[red]Failed to update category {category_id}[/red]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Update category failed: {e}")
        raise click.Abort() from None


@category_command.command(name="delete")
@click.argument("category_id", type=int)
@click.option("--server", default=None, help="Server name from config")
@click.confirmation_option(prompt="Are you sure you want to delete this category?")
def delete_category(category_id, server):
    """
    Delete a category

    Examples:

        # Delete category
        praisonaiwp category delete 123
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        success = wp.delete_category(category_id)

        if success:
            console.print(f"[green]✓ Deleted category {category_id}[/green]")
        else:
            console.print(f"[red]Failed to delete category {category_id}[/red]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Delete category failed: {e}")
        raise click.Abort() from None



"""Tests for meta CLI commands (post and user meta)"""

from unittest.mock import MagicMock, patch

import pytest
from click.testing import CliRunner

from praisonaiwp.cli.commands.meta import meta_command


@pytest.fixture
def runner():
    return CliRunner()


@pytest.fixture
def mock_wp_client():
    with patch('praisonaiwp.cli.commands.meta.WPClient') as mock:
        client = MagicMock()
        mock.return_value = client
        yield client


@pytest.fixture
def mock_ssh():
    with patch('praisonaiwp.cli.commands.meta.SSHManager') as mock:
        yield mock


@pytest.fixture
def mock_config():
    with patch('praisonaiwp.cli.commands.meta.Config') as mock:
        config = MagicMock()
        config.get_server.return_value = {
            'hostname': 'test.com',
            'username': 'testuser',
            'wp_path': '/var/www/html',
            'php_bin': 'php',
            'wp_cli': '/usr/local/bin/wp'
        }
        mock.return_value = config
        yield config


class TestPostMetaGet:
    def test_post_meta_get_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test getting post meta value"""
        mock_wp_client.get_post_meta.return_value = "test_value"

        result = runner.invoke(meta_command, ['post-get', '123', 'custom_field'])

        assert result.exit_code == 0
        assert "test_value" in result.output
        mock_wp_client.get_post_meta.assert_called_once_with(123, 'custom_field')

    def test_post_meta_get_all(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test getting all post meta"""
        mock_wp_client.get_post_meta.return_value = [
            {'meta_key': 'field1', 'meta_value': 'value1'},
            {'meta_key': 'field2', 'meta_value': 'value2'}
        ]

        result = runner.invoke(meta_command, ['post-get', '123'])

        assert result.exit_code == 0
        mock_wp_client.get_post_meta.assert_called_once_with(123, None)


class TestPostMetaSet:
    def test_post_meta_set_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test setting post meta value"""
        mock_wp_client.set_post_meta.return_value = True

        result = runner.invoke(meta_command, ['post-set', '123', 'custom_field', 'test_value'])

        assert result.exit_code == 0
        assert "Set meta" in result.output or "✓" in result.output
        mock_wp_client.set_post_meta.assert_called_once_with(123, 'custom_field', 'test_value')


class TestPostMetaUpdate:
    def test_post_meta_update_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test updating post meta value"""
        mock_wp_client.update_post_meta.return_value = True

        result = runner.invoke(meta_command, ['post-update', '123', 'custom_field', 'new_value'])

        assert result.exit_code == 0
        assert "Updated meta" in result.output or "✓" in result.output
        mock_wp_client.update_post_meta.assert_called_once_with(123, 'custom_field', 'new_value')


class TestPostMetaDelete:
    def test_post_meta_delete_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test deleting post meta"""
        mock_wp_client.delete_post_meta.return_value = True

        result = runner.invoke(meta_command, ['post-delete', '123', 'custom_field'])

        assert result.exit_code == 0
        assert "Deleted meta" in result.output or "✓" in result.output
        mock_wp_client.delete_post_meta.assert_called_once_with(123, 'custom_field')


class TestUserMetaGet:
    def test_user_meta_get_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test getting user meta value"""
        mock_wp_client.get_user_meta.return_value = "test_value"

        result = runner.invoke(meta_command, ['user-get', '456', 'user_field'])

        assert result.exit_code == 0
        assert "test_value" in result.output
        mock_wp_client.get_user_meta.assert_called_once_with(456, 'user_field')

    def test_user_meta_get_all(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test getting all user meta"""
        mock_wp_client.get_user_meta.return_value = [
            {'meta_key': 'field1', 'meta_value': 'value1'},
            {'meta_key': 'field2', 'meta_value': 'value2'}
        ]

        result = runner.invoke(meta_command, ['user-get', '456'])

        assert result.exit_code == 0
        mock_wp_client.get_user_meta.assert_called_once_with(456, None)


class TestUserMetaSet:
    def test_user_meta_set_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test setting user meta value"""
        mock_wp_client.set_user_meta.return_value = True

        result = runner.invoke(meta_command, ['user-set', '456', 'user_field', 'test_value'])

        assert result.exit_code == 0
        assert "Set meta" in result.output or "✓" in result.output
        mock_wp_client.set_user_meta.assert_called_once_with(456, 'user_field', 'test_value')


class TestUserMetaUpdate:
    def test_user_meta_update_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test updating user meta value"""
        mock_wp_client.update_user_meta.return_value = True

        result = runner.invoke(meta_command, ['user-update', '456', 'user_field', 'new_value'])

        assert result.exit_code == 0
        assert "Updated meta" in result.output or "✓" in result.output
        mock_wp_client.update_user_meta.assert_called_once_with(456, 'user_field', 'new_value')


class TestUserMetaDelete:
    def test_user_meta_delete_success(self, runner, mock_config, mock_ssh, mock_wp_client):
        """Test deleting user meta"""
        mock_wp_client.delete_user_meta.return_value = True

        result = runner.invoke(meta_command, ['user-delete', '456', 'user_field'])

        assert result.exit_code == 0
        assert "Deleted meta" in result.output or "✓" in result.output
        mock_wp_client.delete_user_meta.assert_called_once_with(456, 'user_field')



"""
MCP Prompts for WordPress Operations

Prompts are reusable templates that help LLMs perform common WordPress tasks.
They provide structured instructions for content creation and management.
"""

from typing import Dict, Optional


def create_blog_post_prompt(
    topic: str,
    style: str = "professional",
    length: str = "medium",
) -> str:
    """
    Generate a prompt for creating a blog post.

    Args:
        topic: The topic or subject of the blog post
        style: Writing style - 'professional', 'casual', 'technical', 'friendly'
        length: Post length - 'short' (300 words), 'medium' (600 words), 'long' (1200 words)

    Returns:
        Formatted prompt string for the LLM
    """
    length_words = {
        "short": "approximately 300 words",
        "medium": "approximately 600 words",
        "long": "approximately 1200 words",
    }

    style_descriptions = {
        "professional": "professional and authoritative tone, suitable for business audiences",
        "casual": "casual and conversational tone, engaging and easy to read",
        "technical": "technical and detailed, with accurate terminology and explanations",
        "friendly": "warm and friendly tone, approachable and encouraging",
    }

    word_count = length_words.get(length, length_words["medium"])
    style_desc = style_descriptions.get(style, style_descriptions["professional"])

    return f"""Create a WordPress blog post about: {topic}

**Requirements:**
- Length: {word_count}
- Style: {style_desc}
- Include an engaging introduction that hooks the reader
- Use clear headings and subheadings (H2, H3)
- Include a compelling conclusion with a call to action
- Format content with proper HTML tags for WordPress

**Structure:**
1. Introduction (hook the reader)
2. Main content (2-4 sections with headings)
3. Conclusion (summarize and call to action)

**After generating the content, use the create_post tool to publish it.**

Topic: {topic}
"""


def update_content_prompt(
    post_id: int,
    instructions: str,
) -> str:
    """
    Generate a prompt for updating existing content.

    Args:
        post_id: The ID of the post to update
        instructions: Specific instructions for the update

    Returns:
        Formatted prompt string for the LLM
    """
    return f"""Update WordPress post ID {post_id} with the following instructions:

**Instructions:**
{instructions}

**Steps:**
1. First, use the get_post tool to retrieve the current content of post {post_id}
2. Analyze the existing content
3. Apply the requested changes while maintaining the overall structure
4. Use the update_post tool to save the changes

**Guidelines:**
- Preserve the original voice and style unless instructed otherwise
- Maintain SEO-friendly formatting
- Keep existing headings structure if appropriate
- Ensure all links and media references remain intact

Post ID: {post_id}
"""


def bulk_update_prompt(
    operation: str,
    filters: Optional[Dict] = None,
) -> str:
    """
    Generate a prompt for bulk operations.

    Args:
        operation: The operation to perform - 'update_status', 'add_category', 'find_replace'
        filters: Filters to select posts (status, category, date_range, etc.)

    Returns:
        Formatted prompt string for the LLM
    """
    filters = filters or {}
    filter_str = "\n".join([f"- {k}: {v}" for k, v in filters.items()]) or "- No filters (all posts)"

    operation_instructions = {
        "update_status": """
**Operation: Update Post Status**
Change the status of matching posts (e.g., draft to publish, publish to private)
Use the update_post tool for each matching post.""",

        "add_category": """
**Operation: Add Category**
Add a category to matching posts without removing existing categories.
Use the set_post_categories tool for each matching post.""",

        "find_replace": """
**Operation: Find and Replace**
Find and replace text across matching posts.
Use the update_post tool with find_text and replace_text parameters.""",
    }

    instruction = operation_instructions.get(operation, f"**Operation: {operation}**")

    return f"""Perform a bulk operation on WordPress posts.

{instruction}

**Filters to apply:**
{filter_str}

**Steps:**
1. Use list_posts to find all posts matching the filters
2. Review the list of matching posts
3. For each post, apply the operation
4. Report the results

**Safety:**
- Always preview changes before applying
- Process posts in batches if there are many
- Report any errors encountered

Operation: {operation}
"""


def seo_optimize_prompt(post_id: int) -> str:
    """
    Generate a prompt for SEO optimization.

    Args:
        post_id: The ID of the post to optimize

    Returns:
        Formatted prompt string for the LLM
    """
    return f"""Optimize WordPress post ID {post_id} for SEO (Search Engine Optimization).

**Steps:**
1. Use get_post to retrieve the current content of post {post_id}
2. Analyze the content for SEO improvements

**SEO Checklist:**
- [ ] Title tag optimization (include primary keyword, 50-60 characters)
- [ ] Meta description (compelling, 150-160 characters)
- [ ] Heading structure (H1, H2, H3 hierarchy)
- [ ] Keyword placement (title, first paragraph, headings)
- [ ] Internal linking opportunities
- [ ] Image alt text optimization
- [ ] Content length (aim for 1000+ words for comprehensive topics)
- [ ] Readability (short paragraphs, bullet points)
- [ ] Call to action

**Recommendations:**
After analysis, provide specific recommendations for:
1. Title improvements
2. Content structure changes
3. Keyword optimization
4. Meta description suggestion
5. Internal linking suggestions

**Implementation:**
Use update_post to apply approved changes.

Post ID: {post_id}
"""



"""WordPress media management commands"""

import click
from rich.console import Console
from rich.table import Table

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.logger import get_logger

console = Console()
logger = get_logger(__name__)


@click.group()
def media_command():
    """Manage WordPress media"""
    pass


@media_command.command('upload')
@click.argument('file_path')
@click.option('--title', help='Media title')
@click.option('--caption', help='Media caption')
@click.option('--alt-text', help='Alt text for the media')
@click.option('--server', default=None, help='Server name from config')
def upload_media(file_path, title, caption, alt_text, server):
    """
    Upload a media file to WordPress

    Examples:

        # Upload image
        praisonaiwp media upload /path/to/image.jpg

        # Upload with metadata
        praisonaiwp media upload /path/to/image.jpg --title "My Image" --caption "Image caption" --alt-text "Alt text"
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        console.print(f"Uploading media: {file_path}...")
        media_info = wp.media_upload(file_path, title, caption, alt_text)

        if media_info:
            console.print(f"[green]✓ Media uploaded successfully[/green]")
            console.print(f"URL: {media_info.get('url', 'N/A')}")
            console.print(f"ID: {media_info.get('id', 'N/A')}")
            console.print(f"Title: {media_info.get('title', 'N/A')}")
        else:
            console.print(f"[red]✗ Failed to upload media[/red]")
            raise click.ClickException("Media upload failed")

    except click.ClickException:
        raise
    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Upload media failed: {e}")
        raise click.Abort() from None


@media_command.command('delete')
@click.argument('media_id', type=int)
@click.option('--server', default=None, help='Server name from config')
@click.confirmation_option(prompt='Are you sure you want to delete this media file?')
def delete_media(media_id, server):
    """
    Delete a media file from WordPress

    Examples:

        # Delete media
        praisonaiwp media delete 123
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        success = wp.media_delete(media_id)
        if success:
            console.print(f"[green]✓ Deleted media ID: {media_id}[/green]")
        else:
            console.print(f"[red]✗ Failed to delete media ID: {media_id}[/red]")
            raise click.ClickException(f"Media deletion failed for {media_id}")

    except click.ClickException:
        raise
    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Delete media failed: {e}")
        raise click.Abort() from None


@media_command.command('list')
@click.option('--server', default=None, help='Server name from config')
def list_media(server):
    """
    List WordPress media files

    Examples:

        # List all media
        praisonaiwp media list
    """
    try:
        config = Config()
        server_config = config.get_server(server)

        transport = get_transport(config, server)
        transport.connect()
        wp = WPClient(
            transport,
            server_config['wp_path'],
            server_config.get('php_bin', 'php'),
            server_config.get('wp_cli', '/usr/local/bin/wp')
        )


        media_list = wp.list_media()

        if not media_list:
            console.print("[yellow]No media found[/yellow]")
            return

        table = Table(title="WordPress Media")
        table.add_column("ID", style="cyan")
        table.add_column("Title", style="green")
        table.add_column("URL", style="blue")
        table.add_column("Type", style="yellow")

        for media in media_list:
            table.add_row(
                str(media.get('ID', '')),
                media.get('title', ''),
                media.get('url', ''),
                media.get('mime_type', '')
            )

        console.print(table)
        console.print(f"\n[dim]Total: {len(media_list)} media file(s)[/dim]")

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"List media failed: {e}")
        raise click.Abort() from None



"""WordPress term management commands"""

import click
from rich.console import Console
from rich.table import Table

from praisonaiwp.core.config import Config
from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.wp_client import WPClient
from praisonaiwp.utils.logger import get_logger

console = Console()
logger = get_logger(__name__)


def _get_wp_client(server=None):
    """Create WPClient using the appropriate transport (SSH or Kubernetes)."""
    config = Config()
    server_config = config.get_server(server)
    transport = get_transport(config, server)
    transport.connect()
    wp = WPClient(
        transport,
        server_config['wp_path'],
        server_config.get('php_bin', 'php'),
        server_config.get('wp_cli', '/usr/local/bin/wp')
    )
    return transport, wp


@click.group()
def term_command():
    """Manage WordPress taxonomy terms"""
    pass


@term_command.command("list")
@click.argument("taxonomy")
@click.option("--server", default=None, help="Server name from config")
def list_terms(taxonomy, server):
    """
    List WordPress taxonomy terms

    Examples:
    # List all categories
    praisonaiwp term list category

    # List all tags
    praisonaiwp term list post_tag

    # List from specific server
    praisonaiwp term list category --server staging
    """
    try:
        transport, client = _get_wp_client(server)

        terms = client.list_terms(taxonomy)

        if not terms:
            console.print(f"[yellow]No terms found for taxonomy '{taxonomy}'[/yellow]")
            return

        # Create table
        table = Table(title=f"Terms for '{taxonomy}'")
        table.add_column("ID", style="cyan", no_wrap=True)
        table.add_column("Name", style="green")
        table.add_column("Slug", style="yellow")

        for term in terms:
            table.add_row(
                str(term.get('term_id', 'N/A')),
                str(term.get('name', 'N/A')),
                str(term.get('slug', 'N/A'))
            )

        console.print(table)
        transport.close()

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"List terms failed: {e}")
        raise click.Abort() from None


@term_command.command("get")
@click.argument("taxonomy")
@click.argument("term_id")
@click.option("--server", default=None, help="Server name from config")
def get_term(taxonomy, term_id, server):
    """
    Get WordPress taxonomy term information

    Examples:
    # Get term info
    praisonaiwp term get category 1

    # Get from specific server
    praisonaiwp term get post_tag 5 --server staging
    """
    try:
        transport, client = _get_wp_client(server)

        term_info = client.get_term(taxonomy, term_id)

        if term_info:
            console.print("[green]Term Information:[/green]")
            console.print(f"[cyan]ID:[/cyan] {term_info.get('term_id', 'N/A')}")
            console.print(f"[cyan]Name:[/cyan] {term_info.get('name', 'N/A')}")
            console.print(f"[cyan]Slug:[/cyan] {term_info.get('slug', 'N/A')}")
            console.print(f"[cyan]Taxonomy:[/cyan] {term_info.get('taxonomy', 'N/A')}")
            console.print(f"[cyan]Description:[/cyan] {term_info.get('description', 'N/A')}")
        else:
            console.print(f"[yellow]Term '{term_id}' not found in taxonomy '{taxonomy}'[/yellow]")

        transport.close()

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Get term failed: {e}")
        raise click.Abort() from None


@term_command.command("create")
@click.argument("taxonomy")
@click.argument("name")
@click.option("--slug", default=None, help="Term slug")
@click.option("--parent", default=None, help="Parent term ID")
@click.option("--description", default=None, help="Term description")
@click.option("--server", default=None, help="Server name from config")
def create_term(taxonomy, name, slug, parent, description, server):
    """
    Create a WordPress taxonomy term

    Examples:
    # Create a category
    praisonaiwp term create category "Technology"

    # Create a tag with slug
    praisonaiwp term create post_tag "WordPress" --slug wordpress

    # Create a child category
    praisonaiwp term create category "Child Category" --parent 1

    # Create from specific server
    praisonaiwp term create category "News" --server staging
    """
    try:
        transport, client = _get_wp_client(server)

        term_info = client.create_term(taxonomy, name, slug, parent, description)

        if term_info:
            console.print(f"[green]Successfully created term '{name}'[/green]")
            console.print(f"[cyan]ID:[/cyan] {term_info.get('term_id', 'N/A')}")
            console.print(f"[cyan]Slug:[/cyan] {term_info.get('slug', 'N/A')}")
        else:
            console.print(f"[red]Failed to create term '{name}'[/red]")
            raise click.Abort()

        transport.close()

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Create term failed: {e}")
        raise click.Abort() from None


@term_command.command("delete")
@click.argument("taxonomy")
@click.argument("term_id")
@click.option("--server", default=None, help="Server name from config")
def delete_term(taxonomy, term_id, server):
    """
    Delete a WordPress taxonomy term

    Examples:
    # Delete a term
    praisonaiwp term delete category 1

    # Delete from specific server
    praisonaiwp term delete post_tag 5 --server staging
    """
    try:
        transport, client = _get_wp_client(server)

        success = client.delete_term(taxonomy, term_id)

        if success:
            console.print(f"[green]Successfully deleted term '{term_id}' from '{taxonomy}'[/green]")
        else:
            console.print(f"[red]Failed to delete term '{term_id}'[/red]")
            raise click.Abort()

        transport.close()

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        logger.error(f"Delete term failed: {e}")
        raise click.Abort() from None



"""Tests for ServerRouter - automatic server routing based on website URLs"""

from praisonaiwp.core.router import ServerRouter


class TestServerRouter:
    """Test ServerRouter class for auto-routing to correct server"""

    def test_find_server_by_website_exact_match(self, tmp_path):
        """Test finding server by exact website URL"""
        config = {
            'servers': {
                'biblerevelation': {
                    'website': 'https://biblerevelation.org',
                    'hostname': 'christsong.in',
                },
                'default': {
                    'website': 'https://mer.vin',
                    'hostname': 'ionos',
                }
            }
        }

        router = ServerRouter(config)
        server_name, server_config = router.find_server_by_website('https://biblerevelation.org')

        assert server_name == 'biblerevelation'
        assert server_config['hostname'] == 'christsong.in'

    def test_find_server_by_website_without_protocol(self, tmp_path):
        """Test finding server by domain without https://"""
        config = {
            'servers': {
                'biblerevelation': {
                    'website': 'https://biblerevelation.org',
                    'hostname': 'christsong.in',
                }
            }
        }

        router = ServerRouter(config)
        server_name, _ = router.find_server_by_website('biblerevelation.org')

        assert server_name == 'biblerevelation'

    def test_find_server_by_website_with_www(self, tmp_path):
        """Test finding server handles www. prefix correctly"""
        config = {
            'servers': {
                'test': {
                    'website': 'https://example.com',
                    'hostname': 'test.com',
                }
            }
        }

        router = ServerRouter(config)

        # Should match with or without www.
        server_name1, _ = router.find_server_by_website('https://www.example.com')
        server_name2, _ = router.find_server_by_website('https://example.com')

        assert server_name1 == 'test'
        assert server_name2 == 'test'

    def test_find_server_by_website_using_aliases(self, tmp_path):
        """Test finding server by alias domain"""
        config = {
            'servers': {
                'default': {
                    'website': 'https://mer.vin',
                    'aliases': [
                        'https://praison.com',
                        'https://www.praison.com'
                    ],
                    'hostname': 'ionos',
                }
            }
        }

        router = ServerRouter(config)

        # Should find by primary website
        server_name1, _ = router.find_server_by_website('mer.vin')
        assert server_name1 == 'default'

        # Should find by alias
        server_name2, _ = router.find_server_by_website('praison.com')
        assert server_name2 == 'default'

        server_name3, _ = router.find_server_by_website('www.praison.com')
        assert server_name3 == 'default'

    def test_find_server_by_website_not_found(self, tmp_path):
        """Test that None is returned when server not found"""
        config = {
            'servers': {
                'test': {
                    'website': 'https://example.com',
                }
            }
        }

        router = ServerRouter(config)
        server_name, server_config = router.find_server_by_website('https://notfound.com')

        assert server_name is None
        assert server_config is None

    def test_find_server_by_keywords_in_title(self, tmp_path):
        """Test finding server by domain mentioned in text"""
        config = {
            'servers': {
                'biblerevelation': {
                    'website': 'https://biblerevelation.org',
                    'hostname': 'christsong.in',
                }
            }
        }

        router = ServerRouter(config)
        text = "Post for biblerevelation.org about Immanuel"
        server_name, _ = router.find_server_by_keywords(text)

        assert server_name == 'biblerevelation'

    def test_find_server_by_keywords_in_content(self, tmp_path):
        """Test finding server by domain in content"""
        config = {
            'servers': {
                'default': {
                    'website': 'https://mer.vin',
                    'aliases': ['https://praison.com'],
                }
            }
        }

        router = ServerRouter(config)
        text = "Check out my blog at praison.com for more updates"
        server_name, _ = router.find_server_by_keywords(text)

        assert server_name == 'default'

    def test_find_server_by_keywords_case_insensitive(self, tmp_path):
        """Test keyword matching is case-insensitive"""
        config = {
            'servers': {
                'test': {
                    'website': 'https://Example.COM',
                }
            }
        }

        router = ServerRouter(config)
        text = "Visit EXAMPLE.com for details"
        server_name, _ = router.find_server_by_keywords(text)

        assert server_name == 'test'

    def test_find_server_by_keywords_not_found(self, tmp_path):
        """Test that None is returned when no domain found in text"""
        config = {
            'servers': {
                'test': {
                    'website': 'https://example.com',
                }
            }
        }

        router = ServerRouter(config)
        text = "This is a post with no domain mentioned"
        server_name, server_config = router.find_server_by_keywords(text)

        assert server_name is None
        assert server_config is None

    def test_get_server_info_single_server(self, tmp_path):
        """Test getting info for a single server"""
        config = {
            'servers': {
                'biblerevelation': {
                    'website': 'https://biblerevelation.org',
                    'description': 'Bible revelation website',
                    'hostname': 'christsong.in',
                }
            }
        }

        router = ServerRouter(config)
        info = router.get_server_info('biblerevelation')

        assert info['name'] == 'biblerevelation'
        assert info['website'] == 'https://biblerevelation.org'
        assert info['description'] == 'Bible revelation website'
        assert info['hostname'] == 'christsong.in'

    def test_get_server_info_all_servers(self, tmp_path):
        """Test getting info for all servers"""
        config = {
            'servers': {
                'default': {
                    'website': 'https://mer.vin',
                    'description': 'Main blog',
                },
                'biblerevelation': {
                    'website': 'https://biblerevelation.org',
                    'description': 'Bible site',
                }
            }
        }

        router = ServerRouter(config)
        info = router.get_server_info()

        assert 'default' in info
        assert 'biblerevelation' in info
        assert info['default']['website'] == 'https://mer.vin'
        assert info['biblerevelation']['website'] == 'https://biblerevelation.org'

    def test_auto_route_setting_enabled(self, tmp_path):
        """Test that auto_route setting is accessible"""
        config = {
            'servers': {'test': {}},
            'settings': {
                'auto_route': True
            }
        }

        router = ServerRouter(config)

        assert router.auto_route is True

    def test_auto_route_setting_disabled_by_default(self, tmp_path):
        """Test that auto_route defaults to False if not set"""
        config = {
            'servers': {'test': {}},
            'settings': {}
        }

        router = ServerRouter(config)

        assert router.auto_route is False



"""WordPress admin management commands"""
import click

from praisonaiwp.core.transport import get_transport
from praisonaiwp.core.config import Config
from praisonaiwp.core.wp_client import WPClient


@click.group()
def admin():
    """WordPress admin management."""
    pass


@admin.command()
@click.option('--server', help='Server name from config')
def url(server):
    """Get admin URL."""
    config = Config()
    transport = get_transport(config, server)
    transport.connect()
    client = WPClient(ssh, config.get_server(server)['wp_path'] if server else None)
    
    result = client.cli('admin url')
    click.echo(result)


@admin.command()
@click.option('--server', help='Server name from config')
def path(server):
    """Get admin path."""
    config = Config()
    transport = get_transport(config, server)
    transport.connect()
    client = WPClient(ssh, config.get_server(server)['wp_path'] if server else None)
    
    result = client.cli('admin path')
    click.echo(result)


@admin.command()
@click.option('--server', help='Server name from config')
def home(server):
    """Get home URL."""
    config = Config()
    transport = get_transport(config, server)
    transport.connect()
    client = WPClient(ssh, config.get_server(server)['wp_path'] if server else None)
    
    result = client.cli('admin home')
    click.echo(result)


@admin.command()
@click.option('--server', help='Server name from config')
def siteurl(server):
    """Get site URL."""
    config = Config()
    transport = get_transport(config, server)
    transport.connect()
    client = WPClient(ssh, config.get_server(server)['wp_path'] if server else None)
    
    result = client.cli('admin siteurl')
    click.echo(result)


@admin.command()
@click.option('--server', help='Server name from config')
def admin_email(server):
    """Get admin email."""
    config = Config()
    transport = get_transport(config, server)
    transport.connect()
    client = WPClient(ssh, config.get_server(server)['wp_path'] if server else None)
    
    result = client.cli('admin admin-email')
    click.echo(result)
